@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,25 +14,24 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *
- *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * either of the GNU General Public License Version 2 or later (the "GPL"),
+ * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -59,7 +58,8 @@
 #include "nsIParser.h"
 #include "nsIComponentManager.h"
 #include "nsParserCIID.h"
-#include "nsIHTMLFragmentContentSink.h"
+#include "nsIFragmentContentSink.h"
+#include "nsIContentSink.h"
 #include "nsIEnumerator.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsIScriptGlobalObject.h"
@@ -121,29 +121,23 @@ class nsAutoRangeLock
 
 
 
-// Returns -1 if point1 < point2, 1, if point1 > point2,
-// 0 if error or if point1 == point2. 
-PRInt32 ComparePoints(nsIDOMNode* aParent1, PRInt32 aOffset1,
-                      nsIDOMNode* aParent2, PRInt32 aOffset2)
+/* static */
+PRInt32
+nsRange::ComparePoints(nsIDOMNode* aParent1, PRInt32 aOffset1,
+                       nsIDOMNode* aParent2, PRInt32 aOffset2)
 {
-  if (aParent1 == aParent2 && aOffset1 == aOffset2)
-    return 0;
-  nsIDOMRange* range;
-  if (NS_FAILED(NS_NewRange(&range)))
-    return 0;
-  nsresult res = range->SetStart(aParent1, aOffset1);
-  if (NS_FAILED(res))
-    return 0;
-  res = range->SetEnd(aParent2, aOffset2);
-  NS_RELEASE(range);
-  if (NS_SUCCEEDED(res))
-    return -1;
-  else
-    return 1;
+  if (aParent1 == aParent2) {
+    return (aOffset1 < aOffset2) ? -1 :
+      ((aOffset1 > aOffset2) ? 1 : 0);
+  }
+
+  return IsIncreasing(aParent1, aOffset1, aParent2, aOffset2) ? -1 : 1;
 }
 
 // Utility routine to detect if a content node intersects a range
-PRBool IsNodeIntersectsRange(nsIContent* aNode, nsIDOMRange* aRange)
+/* static */
+PRBool
+nsRange::IsNodeIntersectsRange(nsIContent* aNode, nsIDOMRange* aRange)
 {
   // create a pair of dom points that expresses location of node:
   //     NODE(start), NODE(end)
@@ -279,19 +273,15 @@ PRBool GetNodeBracketPoints(nsIContent* aNode,
   if (!outEndOffset)
     return PR_FALSE;
     
-  nsCOMPtr<nsIDOMNode> theDOMNode( do_QueryInterface(aNode) );
-  theDOMNode->GetParentNode(getter_AddRefs(*outParent));
+  nsIContent* parent = aNode->GetParent();
 
-  if (!(*outParent)) // special case for root node
+  if (!parent) // special case for root node
   {
     // can't make a parent/offset pair to represent start or 
     // end of the root node, becasue it has no parent.
     // so instead represent it by (node,0) and (node,numChildren)
     *outParent = do_QueryInterface(aNode);
-    nsCOMPtr<nsIContent> cN(do_QueryInterface(*outParent));
-    if (!cN)
-      return PR_FALSE;
-    PRUint32 indx = cN->GetChildCount();
+    PRUint32 indx = aNode->GetChildCount();
     if (!indx)
       return PR_FALSE;
     *outStartOffset = 0;
@@ -299,7 +289,8 @@ PRBool GetNodeBracketPoints(nsIContent* aNode,
   }
   else
   {
-    *outStartOffset = nsRange::IndexOf(theDOMNode);
+    *outParent = do_QueryInterface(parent);
+    *outStartOffset = parent->IndexOf(aNode);
     *outEndOffset = *outStartOffset+1;
   }
   return PR_TRUE;
@@ -354,15 +345,15 @@ NS_IMPL_ISUPPORTS1(nsRangeUtils, nsIRangeUtils)
  
 NS_IMETHODIMP_(PRInt32) 
 nsRangeUtils::ComparePoints(nsIDOMNode* aParent1, PRInt32 aOffset1,
-                                     nsIDOMNode* aParent2, PRInt32 aOffset2)
+                            nsIDOMNode* aParent2, PRInt32 aOffset2)
 {
-  return ::ComparePoints(aParent1, aOffset1, aParent2, aOffset2);
+  return nsRange::ComparePoints(aParent1, aOffset1, aParent2, aOffset2);
 }
 
 NS_IMETHODIMP_(PRBool) 
 nsRangeUtils::IsNodeIntersectsRange(nsIContent* aNode, nsIDOMRange* aRange)
 {
-  return ::IsNodeIntersectsRange( aNode,  aRange);
+  return nsRange::IsNodeIntersectsRange( aNode,  aRange);
 }
 
 NS_IMETHODIMP
@@ -1597,8 +1588,10 @@ nsresult nsRange::DeleteContents()
 
       node->GetParentNode(getter_AddRefs(parent));
 
-      res = parent->RemoveChild(node, getter_AddRefs(tmpNode));
-      if (NS_FAILED(res)) return res;
+      if (parent) {
+        res = parent->RemoveChild(node, getter_AddRefs(tmpNode));
+        if (NS_FAILED(res)) return res;
+      }
     }
   }
 
@@ -1996,124 +1989,59 @@ nsresult nsRange::InsertNode(nsIDOMNode* aN)
   return tStartContainer->InsertBefore(aN, tChildNode, getter_AddRefs(tResultNode));
 }
 
-nsresult nsRange::SurroundContents(nsIDOMNode* aN)
+nsresult nsRange::SurroundContents(nsIDOMNode* aNewParent)
 {
-  VALIDATE_ACCESS(aN);
+  VALIDATE_ACCESS(aNewParent);
   
-  nsresult res;
+  // Extract the contents within the range.
 
-  //get start offset, and start container
-  PRInt32 tStartOffset;
-  this->GetStartOffset(&tStartOffset);
-  nsCOMPtr<nsIDOMNode> tStartContainer;
-  res = GetStartContainer(getter_AddRefs(tStartContainer));
-  if(NS_FAILED(res)) return res;
+  nsCOMPtr<nsIDOMDocumentFragment> docFrag;
 
-  //get end offset, and end container
-  PRInt32 tEndOffset;
-  this->GetEndOffset(&tEndOffset);
-  nsCOMPtr<nsIDOMNode> tEndContainer;
-  res = GetEndContainer(getter_AddRefs(tEndContainer));
-  if(NS_FAILED(res)) return res;
+  nsresult res = ExtractContents(getter_AddRefs(docFrag));
 
-  //prep start
-  PRUint16 tStartNodeType;
-  tStartContainer->GetNodeType(&tStartNodeType);
-  if( (nsIDOMNode::CDATA_SECTION_NODE == tStartNodeType) ||
-      (nsIDOMNode::TEXT_NODE == tStartNodeType) )
+  if (NS_FAILED(res)) return res;
+  if (!docFrag) return NS_ERROR_FAILURE;
+
+  // Spec says we need to remove all of aNewParent's
+  // children prior to insertion.
+
+  nsCOMPtr<nsIDOMNodeList> children;
+  res = aNewParent->GetChildNodes(getter_AddRefs(children));
+
+  if (NS_FAILED(res)) return res;
+  if (!children) return NS_ERROR_FAILURE;
+
+  PRUint32 numChildren = 0;
+  res = children->GetLength(&numChildren);
+  if (NS_FAILED(res)) return res;
+
+  nsCOMPtr<nsIDOMNode> tmpNode;
+
+  while (numChildren)
   {
-    nsCOMPtr<nsIDOMText> tStartContainerText = do_QueryInterface(tStartContainer);
-    nsCOMPtr<nsIDOMText> tTempText;
-    res = tStartContainerText->SplitText(tStartOffset, getter_AddRefs(tTempText));
-    if(NS_FAILED(res)) return res;
-    tStartOffset = 0;
-    tStartContainer = do_QueryInterface(tTempText);
+    nsCOMPtr<nsIDOMNode> child;
+    res = children->Item(--numChildren, getter_AddRefs(child));
+
+    if (NS_FAILED(res)) return res;
+    if (!child) return NS_ERROR_FAILURE;
+
+    res = aNewParent->RemoveChild(child, getter_AddRefs(tmpNode));
+    if (NS_FAILED(res)) return res;
   }
 
-  //prep end
-  PRUint16 tEndNodeType;
-  tEndContainer->GetNodeType(&tEndNodeType);
-  if( (nsIDOMNode::CDATA_SECTION_NODE == tEndNodeType) ||
-      (nsIDOMNode::TEXT_NODE == tEndNodeType) )
-  {
-    nsCOMPtr<nsIDOMText> tEndContainerText = do_QueryInterface(tEndContainer);
-    nsCOMPtr<nsIDOMText> tTempText;
-    res = tEndContainerText->SplitText(tEndOffset, getter_AddRefs(tTempText));
-    if(NS_FAILED(res)) return res;
+  // Insert aNewParent at the range's start point.
 
-    tEndContainer = do_QueryInterface(tTempText);
-  }
-
-  //get ancestor info
-  nsCOMPtr<nsIDOMNode> tAncestorContainer;
-  this->GetCommonAncestorContainer(getter_AddRefs(tAncestorContainer));
-
-  PRUint16 tCommonAncestorType;
-  tAncestorContainer->GetNodeType(&tCommonAncestorType);
-
-  nsCOMPtr<nsIDOMNode>tempNode;
-  nsCOMPtr<nsIDOMNode>tRangeContentsNode;
-  nsCOMPtr<nsIDOMDocument> document;
-  res = mStartParent->GetOwnerDocument(getter_AddRefs(document));
+  res = InsertNode(aNewParent);
   if (NS_FAILED(res)) return res;
 
-  // Create a new document fragment in the context of this document,
-  // which might be null
-  nsCOMPtr<nsIDOMDocumentFragment> docfrag;
+  // Append the content we extracted under aNewParent.
 
-  nsCOMPtr<nsIDocument> doc(do_QueryInterface(document));
-
-  res = NS_NewDocumentFragment(getter_AddRefs(docfrag), doc);
+  res = aNewParent->AppendChild(docFrag, getter_AddRefs(tmpNode));
   if (NS_FAILED(res)) return res;
 
-  res = this->ExtractContents(getter_AddRefs(docfrag));
-  if (NS_FAILED(res)) return res;
+  // Select aNewParent, and its contents.
 
-  tRangeContentsNode = do_QueryInterface(docfrag);
-  aN->AppendChild(tRangeContentsNode, getter_AddRefs(tempNode));
-
-  if( (nsIDOMNode::CDATA_SECTION_NODE == tCommonAncestorType) ||
-      (nsIDOMNode::TEXT_NODE == tCommonAncestorType) )
-  {//easy stuff here
-    this->InsertNode(aN);
-  }
-  else
-  {//hard stuff here
-    nsCOMPtr<nsIDOMNodeList>tChildList;
-    res = tAncestorContainer->GetChildNodes(getter_AddRefs(tChildList));
-    PRUint32 i,tNumChildren;
-    tChildList->GetLength(&tNumChildren);
-
-    PRBool tFound = PR_FALSE;
-    PRInt16 tResult;
-    for(i = 0; (i < tNumChildren && !tFound); i++)
-    {
-      ComparePoint(tAncestorContainer, i, &tResult);
-      if(tResult == 0)
-      {
-        tFound = PR_TRUE;
-        break;
-      }
-    }
-
-    if(tFound)
-    {
-      nsCOMPtr<nsIDOMNode> tChild;
-      tChildList->Item(i, getter_AddRefs(tChild));
-      tAncestorContainer->InsertBefore(aN, tChild, getter_AddRefs(tempNode));
-    }
-    else // there is an error this may need to be updated later
-      this->InsertNode(aN);
-
-    // re-define the range so that it contains the same content as it did before
-    tEndOffset = GetNodeLength(tEndContainer);
-    if (tEndOffset == -1)  // failure code
-      return NS_ERROR_FAILURE;
-      
-    this->DoSetRange(tStartContainer, 0, tEndContainer, tEndOffset);
-  }
-  this->SelectNode(aN);
-  return NS_OK;
+  return SelectNode(aNewParent);
 }
 
 nsresult nsRange::ToString(nsAString& aReturn)
@@ -2343,22 +2271,22 @@ nsresult nsRange::OwnerChildReplaced(nsIContent* aParentNode, PRInt32 aOffset, n
 }
   
 
-nsresult nsRange::TextOwnerChanged(nsIContent* aTextNode, PRInt32 aStartChanged, PRInt32 aEndChanged, PRInt32 aReplaceLength)
+nsresult
+nsRange::TextOwnerChanged(nsIContent* aTextNode, nsVoidArray *aRangeList,
+                          PRInt32 aStartChanged, PRInt32 aEndChanged,
+                          PRInt32 aReplaceLength)
 {
-  // sanity check - null nodes shouldn't have enclosed ranges
-  if (!aTextNode) return NS_ERROR_UNEXPECTED;
+  NS_ASSERTION(aRangeList,
+               "Don't call TextOwnerChanged if aTextNode is not in a range!");
+  NS_ASSERTION(aTextNode, "Null nodes don't have enclosed ranges!");
 
-  nsCOMPtr<nsIContent> textNode( do_QueryInterface(aTextNode) );
-  const nsVoidArray *theRangeList = aTextNode->GetRangeList();
-  // the caller already checked to see if there was a range list
-  
-  nsCOMPtr<nsIDOMNode> domNode(do_QueryInterface(textNode));
+  nsCOMPtr<nsIDOMNode> domNode(do_QueryInterface(aTextNode));
   if (!domNode) return NS_ERROR_UNEXPECTED;
 
-  PRInt32   count = theRangeList->Count();
+  PRInt32   count = aRangeList->Count();
   for (PRInt32 loop = 0; loop < count; loop++)
   {
-    nsRange* theRange = NS_STATIC_CAST(nsRange*, (theRangeList->ElementAt(loop))); 
+    nsRange* theRange = NS_STATIC_CAST(nsRange*, (aRangeList->ElementAt(loop))); 
     NS_ASSERTION(theRange, "oops, no range");
 
     // sanity check - do range and content agree over ownership?
@@ -2432,8 +2360,57 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
     
     parent->GetNodeType(&nodeType);
     if (nsIDOMNode::ELEMENT_NODE == nodeType) {
-      nsAutoString tagName;
+      PRInt32 namespaceID;
+      nsAutoString tagName, uriStr;
       parent->GetNodeName(tagName);
+
+      // see if we need to add xmlns declarations
+      nsCOMPtr<nsIContent> content( do_QueryInterface(parent) );
+      PRUint32 count = content->GetAttrCount();
+      PRBool setDefaultNamespace = PR_FALSE;
+      if (count > 0) {
+        PRUint32 index;
+        nsAutoString nameStr, prefixStr, valueStr;
+        nsCOMPtr<nsIAtom> attrName, attrPrefix;
+
+        for (index = 0; index < count; index++) {
+    
+          content->GetAttrNameAt(index,
+                                &namespaceID,
+                                getter_AddRefs(attrName),
+                                getter_AddRefs(attrPrefix));
+    
+          if (namespaceID == kNameSpaceID_XMLNS) {
+            content->GetAttr(namespaceID, attrName, uriStr);
+
+            // really want something like nsXMLContentSerializer::SerializeAttr()
+            tagName.Append(NS_LITERAL_STRING(" xmlns")); // space important
+            if (attrPrefix) {
+              tagName.Append(PRUnichar(':'));
+              attrName->ToString(nameStr);
+              tagName.Append(nameStr);
+            }
+            else {
+              setDefaultNamespace = PR_TRUE;
+            }
+            tagName.Append(NS_LITERAL_STRING("=\"") + uriStr + NS_LITERAL_STRING("\""));
+          }
+        }
+      }
+      if (!setDefaultNamespace) {
+        nsINodeInfo* info = content->GetNodeInfo();
+        if (info && !info->GetPrefixAtom() &&
+            info->NamespaceID() != kNameSpaceID_None) {
+          // We have no namespace prefix, but have a namespace ID.  Push
+          // default namespace attr in, so that our kids will be in our
+          // namespace.
+          nsAutoString uri;
+          info->GetNamespaceURI(uri);
+          tagName.Append(NS_LITERAL_STRING(" xmlns=\"") + uri +
+                         NS_LITERAL_STRING("\""));
+        }
+      }
+
       // XXX Wish we didn't have to allocate here
       PRUnichar* name = ToNewUnicode(tagName);
       if (name) {
@@ -2453,22 +2430,29 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
 
   if (NS_SUCCEEDED(result)) {
     nsCAutoString contentType;
-    nsCOMPtr<nsIHTMLFragmentContentSink> sink;
+    PRBool bCaseSensitive = PR_TRUE;
+    if (document) {
+      nsAutoString buf;
+      document->GetContentType(buf);
+      CopyUCS2toASCII(buf, contentType);
+      bCaseSensitive = document->IsCaseSensitive();
+    }
+    else {
+      contentType.AssignLiteral("text/xml");
+    }
 
-    result = NS_NewHTMLFragmentContentSink(getter_AddRefs(sink));
+    nsCOMPtr<nsIHTMLDocument> htmlDoc(do_QueryInterface(domDocument));
+    PRBool bHTML = htmlDoc && !bCaseSensitive;
+    nsCOMPtr<nsIFragmentContentSink> sink;
+    if (bHTML) {
+      result = NS_NewHTMLFragmentContentSink(getter_AddRefs(sink));
+    } else {
+      result = NS_NewXMLFragmentContentSink(getter_AddRefs(sink));
+    }
     if (NS_SUCCEEDED(result)) {
       sink->SetTargetDocument(document);
-      parser->SetContentSink(sink);
-      nsCOMPtr<nsIDOMNSDocument> domnsDocument(do_QueryInterface(document));
-      if (domnsDocument) {
-        nsAutoString buf;
-        domnsDocument->GetContentType(buf);
-        CopyUCS2toASCII(buf, contentType);
-      }
-      else {
-        // Who're we kidding. This only works for html.
-        contentType = NS_LITERAL_CSTRING("text/html");
-      }
+      nsCOMPtr<nsIContentSink> contentsink( do_QueryInterface(sink) );
+      parser->SetContentSink(contentsink);
 
       // If there's no JS or system JS running,
       // push the current document's context on the JS context stack
@@ -2511,7 +2495,7 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
 
       nsDTDMode mode = eDTDMode_autodetect;
       nsCOMPtr<nsIHTMLDocument> htmlDoc(do_QueryInterface(domDocument));
-      if (htmlDoc) {
+      if (bHTML) {
         switch (htmlDoc->GetCompatibilityMode()) {
           case eCompatibility_NavQuirks:
             mode = eDTDMode_quirks;
@@ -2526,10 +2510,12 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
             NS_NOTREACHED("unknown mode");
             break;
         }
+      } else {
+        mode = eDTDMode_full_standards;
       }
       result = parser->ParseFragment(aFragment, (void*)0,
                                      tagStack,
-                                     0, contentType, mode);
+                                     !bHTML, contentType, mode);
 
       if (ContextStack) {
         JSContext *notused;
@@ -2537,7 +2523,7 @@ nsRange::CreateContextualFragment(const nsAString& aFragment,
       }
 
       if (NS_SUCCEEDED(result)) {
-        sink->GetFragment(aReturn);
+        result = sink->GetFragment(aReturn);
       }
     }
   }
