@@ -1,28 +1,46 @@
 # -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
-# The contents of this file are subject to the Netscape Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/NPL/
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
 #
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
+#
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
 #
 # The Original Code is Mozilla Communicator client code, released
 # March 31, 1998.
 #
-# The Initial Developer of the Original Code is Netscape
-# Communications Corporation. Portions created by Netscape are
-# Copyright (C) 1998-1999 Netscape Communications Corporation. All
-# Rights Reserved.
+# The Initial Developer of the Original Code is
+# Netscape Communications Corporation.
+# Portions created by the Initial Developer are Copyright (C) 1998-1999
+# the Initial Developer. All Rights Reserved.
 #
-# Contributors: timeless
-#               slucy@objectivesw.co.uk
-#               Hkan Waara <hwaara@chello.se>
-#               Jan Varga <varga@nixcorp.com>
-#               Seth Spitzer <sspitzer@netscape.com>
-#               David Bienvenu <bienvenu@netscape.com>
+# Contributor(s):
+#   timeless
+#   slucy@objectivesw.co.uk
+#   Hkan Waara <hwaara@chello.se>
+#   Jan Varga <varga@nixcorp.com>
+#   Seth Spitzer <sspitzer@netscape.com>
+#   David Bienvenu <bienvenu@netscape.com>
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
 
 const MSG_FLAG_READ              = 0x000001;
 const MSG_FLAG_IMAP_DELETED      = 0x200000;
@@ -44,6 +62,11 @@ const kNoRemoteContentPolicy = 0;
 const kBlockRemoteContent = 1;
 const kAllowRemoteContent = 2;
 
+const kMsgNotificationNoStatus = 0;
+const kMsgNotificationJunkBar = 1;
+const kMsgNotificationRemoteImages = 2;
+const kMsgNotificationPhishingBar = 3;
+
 var gMessengerBundle;
 var gPromptService;
 var gOfflinePromptsBundle;
@@ -106,7 +129,7 @@ function menu_new_init()
   EnableMenuItem("menu_newFolder", !isIMAPFolder || !ioService.offline);
   EnableMenuItem("menu_newVirtualFolder", true);
   if (showNew)
-    SetMenuItemLabel("menu_newFolder", gMessengerBundle.getString((isServer || isInbox) ? "newFolder" : "newSubfolder"));
+    SetMenuItemLabel("menu_newFolder", gMessengerBundle.getString((isServer || isInbox) ? "newFolderMenuItem" : "newSubfolderMenuItem"));
 }
 
 function goUpdateMailMenuItems(commandset)
@@ -309,9 +332,10 @@ function viewRefreshCustomMailViews(aCurrentViewValue)
   // remove any existing entries...
   var menupopupNode = document.getElementById('viewMessageViewPopup');
   var userDefinedItems = menupopupNode.getElementsByAttribute("userdefined","true");
-  for (var i=0; i<userDefinedItems.length; i++)
+  for (var i=0; userDefinedItems.item(i); )
   {
-    menupopupNode.removeChild(userDefinedItems[i]);
+    if (!menupopupNode.removeChild(userDefinedItems[i]))
+      ++i;
   }
   
   // now rebuild the list
@@ -321,7 +345,7 @@ function viewRefreshCustomMailViews(aCurrentViewValue)
   for (i = 0; i < numItems; i++)
   {
     var newMenuItem = document.createElement("menuitem");
-    newMenuItem.setAttribute("label", mailViewList.getMailViewAt(i).mailViewName);
+    newMenuItem.setAttribute("label", mailViewList.getMailViewAt(i).prettyName);
     newMenuItem.setAttribute("userdefined", "true");
     var oncommandStr = "ViewMessagesBy('userdefinedview" + (kLastDefaultViewIndex + i) + "');";  
     newMenuItem.setAttribute("oncommand", oncommandStr);
@@ -516,32 +540,8 @@ function InitMessageLabel(menuType)
 
     try
     {
-        var msgFolder = GetLoadedMsgFolder();
-        var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
-        var numSelected = GetNumSelectedMessages();
-        var indices = GetSelectedIndices(gDBView);
         var isChecked = true;
-        var checkedLabel;
-        var msgKey;
-
-        if (numSelected > 0) {
-            msgKey = gDBView.getKeyAt(indices[0]);
-            checkedLabel = msgDatabase.GetMsgHdrForKey(msgKey).label;
-            if (numSelected > 1) {
-                for (var i = 1; i < indices.length; i++)
-                {
-                    msgKey = gDBView.getKeyAt(indices[i]);
-                    if (msgDatabase.GetMsgHdrForKey(msgKey).label == checkedLabel) {
-                        continue;
-                    }
-                    isChecked = false;
-                    break;
-                }
-            }
-        }
-        else {
-            isChecked = false;
-        }
+        var checkedLabel = gDBView.hdrForFirstSelectedMessage.label;
     }
     catch(ex)
     {
@@ -746,7 +746,6 @@ function MsgGetMessagesForAllServers(defaultServer)
     try
     {
         var allServers = accountManager.allServers;
-
         // array of isupportsarrays of servers for a particular folder
         var pop3DownloadServersArray = new Array; 
         // parallel isupports array of folders to download to...
@@ -761,18 +760,17 @@ function MsgGetMessagesForAllServers(defaultServer)
             {
                 if (defaultServer && defaultServer.equals(currentServer) && 
                   !defaultServer.isDeferredTo &&
-                  defaultServer.msgFolder == defaultServer.rootMsgFolder)
+                  defaultServer.rootFolder == defaultServer.rootMsgFolder)
                 {
                     dump(currentServer.serverURI + "...skipping, already opened\n");
                 }
-                else
+                else if (currentServer.type == "pop3" && currentServer.downloadOnBiff)
                 {
-                    if (currentServer.type == "pop3" && currentServer.downloadOnBiff)
-                    {
-                      CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadServersArray, localFoldersToDownloadTo);
-                      pop3Server = currentServer.QueryInterface(Components.interfaces.nsIPop3IncomingServer);
-                    }
-                    else
+                    CoalesceGetMsgsForPop3ServersByDestFolder(currentServer, pop3DownloadServersArray, localFoldersToDownloadTo);
+                    pop3Server = currentServer.QueryInterface(Components.interfaces.nsIPop3IncomingServer);
+                }
+                else
+                {		 
                     // Check to see if there are new messages on the server
                     currentServer.PerformBiff(msgWindow);
                 }
@@ -911,12 +909,17 @@ function MsgNewMessage(event)
 function MsgReplyMessage(event)
 {
   var loadedFolder = GetLoadedMsgFolder();
-  var server = loadedFolder.server;
+  if (loadedFolder)
+  {
+    var server = loadedFolder.server;
 
-  if(server && server.type == "nntp")
-    MsgReplyGroup(event);
-  else
-    MsgReplySender(event);
+    if(server && server.type == "nntp")
+    {
+      MsgReplyGroup(event);
+      return;
+    }
+  }
+  MsgReplySender(event);
 }
 
 function MsgReplySender(event)
@@ -946,10 +949,9 @@ function MsgReplyToAllMessage(event)
   var loadedFolder = GetLoadedMsgFolder();
   var messageArray = GetSelectedMessages();
 
-  if (event && event.shiftKey)
-    ComposeMessage(msgComposeType.ReplyAll, msgComposeFormat.OppositeOfDefault, loadedFolder, messageArray);
-  else
-    ComposeMessage(msgComposeType.ReplyAll, msgComposeFormat.Default, loadedFolder, messageArray);
+  ComposeMessage(msgComposeType.ReplyAll, 
+    (event && event.shiftKey) ? msgComposeFormat.OppositeOfDefault : msgComposeFormat.Default,
+    loadedFolder, messageArray);
 }
 
 function MsgForwardMessage(event)
@@ -1252,9 +1254,9 @@ function MsgOpenFromFile()
    }
 
    var uri = fp.fileURL;
-   uri.query = "type=x-message-display";
+   uri.query = "type=application/x-message-display";
 
-  MsgOpenNewWindowForMessage(uri, null);
+  window.openDialog( "chrome://messenger/content/messageWindow.xul", "_blank", "all,chrome,dialog=no,status,toolbar", uri, null, null );
 }
 
 function MsgOpenNewWindowForMessage(messageUri, folderUri)
@@ -1339,6 +1341,7 @@ function MsgDownloadSelected()
 
 function MsgMarkThreadAsRead()
 {
+  ClearPendingReadTimer();
   gDBView.doCommand(nsMsgViewCommandType.markThreadRead);
 }
 
@@ -1432,7 +1435,7 @@ function MsgFilters(emailAddress, folder)
          we check this here in args to show filterList dialog */
       if ("refresh" in args && args.refresh)
       {
-         args = { refresh: true, folder: preselectedFolder };
+         args = { refresh: true, folder: folder };
          MsgFilterList(args);
       }
     }
@@ -1667,7 +1670,7 @@ function IsGetNextNMessagesEnabled()
     var serverType = server.type;
 
     var menuItem = document.getElementById("menu_getnextnmsg");
-    if((serverType == "nntp")) {
+    if ((serverType == "nntp") && !folder.isServer) {
         var newsServer = server.QueryInterface(Components.interfaces.nsINntpIncomingServer);
         var menuLabel = gMessengerBundle.getFormattedString("getNextNMessages",
                                                             [ newsServer.maxArticles ]);
@@ -1675,10 +1678,9 @@ function IsGetNextNMessagesEnabled()
         menuItem.removeAttribute("hidden");
         return true;
     }
-    else {
-        menuItem.setAttribute("hidden","true");
-        return false;
-    }
+
+    menuItem.setAttribute("hidden","true");
+    return false;
 }
 
 function IsEmptyTrashEnabled()
@@ -1931,7 +1933,7 @@ function GetFolderMessages()
     // should shift click get mail for all (authenticated) accounts?
     // see bug #125885
     if (!folder.server.isDeferredTo)
-     folder = defaultAccountRootFolder;
+      folder = defaultAccountRootFolder;
   }
 
   var folders = new Array(1);
@@ -2109,7 +2111,7 @@ function HandleJunkStatusChanged(folder)
   // this might be the stand alone window, open to a message that was
   // and attachment (or on disk), in which case, we want to ignore it.
   var loadedMessage = GetLoadedMessage();
-  if (loadedMessage && (!(/type=x-message-display/.test(loadedMessage))) && IsCurrentLoadedFolder(folder))
+  if (loadedMessage && (!(/type=application\/x-message-display/.test(loadedMessage))) && IsCurrentLoadedFolder(folder))
   {
     // if multiple message are selected
     // and we change the junk status
@@ -2126,7 +2128,7 @@ function HandleJunkStatusChanged(folder)
         // We have no way of determining if the junk status of our current message has really changed
         // the only thing we can do is cheat by asking if the junkbar visibility had to change as a result of this notification
 
-        var changedJunkStatus = SetUpJunkBar(msgHdr);
+        var changedJunkStatus = gMessageNotificationBar.setJunkMsg(msgHdr);
 
         // we may be forcing junk mail to be rendered with sanitized html. In that scenario, we want to 
         // reload the message if the status has just changed to not junk. 
@@ -2146,58 +2148,81 @@ function HandleJunkStatusChanged(folder)
           // Furthermore, if we are about to move the message that was just marked as junk, 
           // then don't bother reloading it.
           if (!(isJunk && moveJunkMail)) 
-            MsgReload();
+          MsgReload();
         }
       }
     }
     else
-      SetUpJunkBar(null);
+      gMessageNotificationBar.setJunkMsg(null);
   }
 }
 
-// returns true if we actually changed the visiblity of the junk bar otherwise false
-function SetUpJunkBar(aMsgHdr)
+var gMessageNotificationBar = 
 {
-  // XXX todo
-  // should this happen on the start, or at the end?
-  // if at the end, we might keep the "this message is junk" up for a while, until a big message is loaded
-  // or do we need to wait until here, to make sure the message is fully analyzed
-  // what about almost hiding it on the start, and then showing here?
+  mMsgNotificationBar: document.getElementById('msgNotificationBar'),
 
-  var isJunk = false;
+  setJunkMsg: function (aMsgHdr)
+  {
+    var isJunk = false;
+    var isCurrentlyNotJunk = this.mMsgNotificationBar.selectedIndex != kMsgNotificationJunkBar;
   
-  if (aMsgHdr) {
-    var junkScore = aMsgHdr.getStringProperty("junkscore"); 
-    isJunk = ((junkScore != "") && (junkScore != "0"));
+    if (aMsgHdr) 
+    {
+      var junkScore = aMsgHdr.getStringProperty("junkscore"); 
+      isJunk = ((junkScore != "") && (junkScore != "0"));
+    }
+
+    this.updateMsgNotificationBar (isJunk ? kMsgNotificationJunkBar : kMsgNotificationNoStatus);
+
+    goUpdateCommand('button_junk');
+
+    return (isJunk && isCurrentlyNotJunk) || (!isJunk && !isCurrentlyNotJunk);
+  },
+
+  setRemoteContentMsg: function (aMsgHdr)
+  {  
+    // The phishing message and junk message takes precedence over the remote content msg
+    if (this.mMsgNotificationBar.selectedIndex != kMsgNotificationJunkBar && 
+        this.mMsgNotificationBar.selectedIndex != kMsgNotificationPhishingBar)
+      this.updateMsgNotificationBar(aMsgHdr && aMsgHdr.getUint32Property("remoteContentPolicy") == kBlockRemoteContent ? 
+                                    kMsgNotificationRemoteImages : kMsgNotificationNoStatus);
+  },
+
+  // aUrl is the nsIURI for the message currently loaded in the message pane
+  setPhishingMsg: function(aUrl)
+  {
+    var msgURI = GetLoadedMessage();
+    if (msgURI && !(/type=application\/x-message-display/.test(msgURI)))
+    {
+      var msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
+      // if we've explicitly marked this message as not being an email scam, then don't
+      // bother checking it with the phishing detector.
+      if (msgHdr && msgHdr.getUint32Property("notAPhishMessage"))
+        return; 
+    }
+
+    // The Junk message takes precedence over the phishing message...so skip this step
+    // if the message is already marked as junk
+    if (this.mMsgNotificationBar.selectedIndex != kMsgNotificationJunkBar && isMsgEmailScam(aUrl))
+      this.updateMsgNotificationBar(kMsgNotificationPhishingBar);
+  },
+
+  clearMsgNotifications: function()
+  {
+    this.updateMsgNotificationBar(kMsgNotificationNoStatus);
+  },
+
+  // private method used to set our message notification deck to the correct value...
+  updateMsgNotificationBar: function(aIndex)
+  {
+    if (aIndex == kMsgNotificationNoStatus)
+      this.mMsgNotificationBar.setAttribute('collapsed', true);
+    else
+      this.mMsgNotificationBar.removeAttribute('collapsed');
+    
+    this.mMsgNotificationBar.selectedIndex = aIndex;
   }
-  
-  var junkBar = document.getElementById("junkBar");
-  var isAlreadyCollapsed = junkBar.getAttribute("collapsed") == "true";
-
-  if (isJunk)
-    junkBar.removeAttribute("collapsed");
-  else
-    junkBar.setAttribute("collapsed","true");
- 
-  goUpdateCommand('button_junk');
-
-  return (isJunk && isAlreadyCollapsed) || (!isJunk && !isAlreadyCollapsed);
-}
-
-// hides or shows the remote content bar based on the property in the msg hdr
-function SetUpRemoteContentBar(aMsgHdr)
-{
-  var showRemoteContentBar = false;
-  if (aMsgHdr && aMsgHdr.getUint32Property("remoteContentPolicy") == kBlockRemoteContent)
-    showRemoteContentBar = true;
-
-  var remoteContentBar = document.getElementById("remoteContentBar");
-  
-  if (showRemoteContentBar)
-    remoteContentBar.removeAttribute("collapsed");
-  else
-    remoteContentBar.setAttribute("collapsed","true");
-}
+};
 
 function LoadMsgWithRemoteContent()
 {
@@ -2208,7 +2233,7 @@ function LoadMsgWithRemoteContent()
   var msgURI = GetLoadedMessage();
   var msgHdr = null;
     
-  if (msgURI && !(/type=x-message-display/.test(msgURI)))
+  if (msgURI && !(/type=application\/x-message-display/.test(msgURI)))
   {
     msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
     if (msgHdr)
@@ -2219,8 +2244,29 @@ function LoadMsgWithRemoteContent()
   }
 }
 
+function MsgIsNotAScam()
+{
+  // we want to get the msg hdr for the currently selected message
+  // change the "isPhishingMsg" property on it
+  // then reload the message
+
+  var msgURI = GetLoadedMessage();
+  var msgHdr = null;
+    
+  if (msgURI && !(/type=application\/x-message-display/.test(msgURI)))
+  {
+    msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
+    if (msgHdr)
+    {
+      msgHdr.setUint32Property("notAPhishMessage", 1); 
+      MsgReload();
+    }
+  }
+}
+
 function MarkCurrentMessageAsRead()
 {
+  ClearPendingReadTimer();
   gDBView.doCommand(nsMsgViewCommandType.markMessagesRead);
 }
 
@@ -2239,6 +2285,8 @@ function OnMsgParsed(aUrl)
 {
   if ("onQuickSearchNewMsgLoaded" in this)
     onQuickSearchNewMsgLoaded();
+  
+  gMessageNotificationBar.setPhishingMsg(aUrl);
 }
 
 function OnMsgLoaded(aUrl)
@@ -2257,12 +2305,17 @@ function OnMsgLoaded(aUrl)
     // if the user clicks on another message then that message stays selected
     // and the selection does not "snap back" to the message chosen by
     // SetNextMessageAfterDelete() when the operation completes (bug 243532).
-    gNextMessageViewIndexAfterDelete = -2;
+    // But the just loaded message might be getting deleted, if the user
+    // deletes it before the message is loaded (bug 183394)
 
-    if (!(/type=x-message-display/.test(msgURI)))
+    var wintype = document.documentElement.getAttribute('windowtype');
+    if (wintype == "mail:messageWindow" || GetThreadTree().view.selection.currentIndex != gSelectedIndexWhenDeleting)
+      gNextMessageViewIndexAfterDelete = -2;
+
+    if (!(/type=application\/x-message-display/.test(msgURI)))
       msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
-        
-      SetUpJunkBar(msgHdr);
+     
+    gMessageNotificationBar.setJunkMsg(msgHdr);
 
     // we just finished loading a message. set a timer to actually mark the message is read after n seconds
     // where n can be configured by the user.
@@ -2271,9 +2324,11 @@ function OnMsgLoaded(aUrl)
 
     if (msgHdr && !msgHdr.isRead)
     {
-      var wintype = document.firstChild.getAttribute('windowtype');
       if (markReadOnADelay && wintype == "mail:3pane") // only use the timer if viewing using the 3-pane preview pane and the user has set the pref
+      {
+        ClearPendingReadTimer();
         gMarkViewedMessageAsReadTimer = setTimeout(MarkCurrentMessageAsRead, gPrefBranch.getIntPref("mailnews.mark_message_read.delay.interval") * 1000);
+      }
       else
         MarkCurrentMessageAsRead();
     }
@@ -2318,7 +2373,7 @@ function OnMsgLoaded(aUrl)
 
         res = outputPFC.copyMessages(currentMsgFolder, messages, false /*isMove*/, msgWindow /* nsIMsgWindow */, null /* listener */, false /* isFolder */, false /*allowUndo*/ );
       }
-     }
+   }
 }
 
 //
@@ -2347,11 +2402,13 @@ function HandleMDNResponse(aUrl)
 
   var msgHdr = messenger.messageServiceFromURI(msgURI).messageURIToMsgHdr(msgURI);
   var mimeHdr;
-  
+
   try {
-    mimeHdr = aUrl.mimeHeaders;  
-  } catch (ex) { return 0;}
-    
+    mimeHdr = aUrl.mimeHeaders;
+  } catch (ex) {
+    return;
+  }
+
   // If we didn't get the message id when we downloaded the message header,
   // we cons up an md5: message id. If we've done that, we'll try to extract
   // the message id out of the mime headers for the whole message.
@@ -2470,12 +2527,79 @@ function OpenOrFocusWindow(args, windowType, chromeURL)
     window.openDialog(chromeURL, "", "chrome,resizable,status,centerscreen,dialog=no", args);
 }
 
+/**
+ * Opens the update manager and checks for updates to the application.
+ */
 
-function loadThrobberUrl(urlPref)
+function checkForUpdates()
 {
-    var url;
-    try {
-        url = gPrefBranch.getComplexValue(urlPref, Components.interfaces.nsIPrefLocalizedString).data;
-        messenger.launchExternalURL(url);  
-    } catch (ex) {}
+  var um = 
+      Components.classes["@mozilla.org/updates/update-manager;1"].
+      getService(Components.interfaces.nsIUpdateManager);
+  var prompter = 
+      Components.classes["@mozilla.org/updates/update-prompt;1"].
+      createInstance(Components.interfaces.nsIUpdatePrompt);
+
+  // If there's an update ready to be applied, show the "Update Downloaded"
+  // UI instead and let the user know they have to restart the application for
+  // the changes to be applied. 
+  if (um.activeUpdate && um.activeUpdate.state == "pending")
+    prompter.showUpdateDownloaded(um.activeUpdate);
+  else
+    prompter.checkForUpdates();
+}
+
+function buildHelpMenu()
+{
+  var updates = 
+      Components.classes["@mozilla.org/updates/update-service;1"].
+      getService(Components.interfaces.nsIApplicationUpdateService);
+  var um = 
+      Components.classes["@mozilla.org/updates/update-manager;1"].
+      getService(Components.interfaces.nsIUpdateManager);
+  
+  // Disable the UI if the update enabled pref has been locked by the
+  // administrator or if we cannot update for some other reason 
+  var checkForUpdates = document.getElementById("checkForUpdates");
+  var canUpdate = updates.canUpdate;
+  checkForUpdates.setAttribute("disabled", !canUpdate);
+  if (!canUpdate)
+    return;
+
+  if (!gMessengerBundle)
+    gMessengerBundle = document.getElementById("bundle_messenger");
+
+  var activeUpdate = um.activeUpdate;
+  
+  // If there's an active update, substitute its name into the label
+  // we show for this item, otherwise display a generic label.
+  function getStringWithUpdateName(key) {
+    if (activeUpdate && activeUpdate.name)
+      return gMessengerBundle.getFormattedString(key, [activeUpdate.name]);
+    return gMessengerBundle.getString(key + "Fallback");
+  }
+  
+  // By default, show "Check for Updates..."
+  var key = "default";
+  if (activeUpdate) {
+    switch (activeUpdate.state) {
+    case "downloading":
+      // If we're downloading an update at present, show the text:
+      // "Downloading Firefox x.x..." otherwise we're paused, and show
+      // "Resume Downloading Firefox x.x..."
+      key = updates.isDownloading ? "downloading" : "resume";
+      break;
+    case "pending":
+      // If we're waiting for the user to restart, show: "Apply Downloaded
+      // Updates Now..."
+      key = "pending";
+      break;
+    }
+  }
+
+  checkForUpdates.label = getStringWithUpdateName("updatesItem_" + key);
+  if (um.activeUpdate && updates.isDownloading)
+    checkForUpdates.setAttribute("loading", "true");
+  else
+    checkForUpdates.removeAttribute("loading");
 }
